#!/usr/bin/env python

# ZConfig 2.2 configuration writer - v.0.1 20041206
# Nicola Larosa <nico@tekNico.net>

INDENT = '  '

import os, os.path
import ZConfig

def sectionLines(sect, indent = 0):
    cfgLines = []
    for key, subInfo in sect.getSectionDefinition():
        value = getattr(sect, subInfo.attribute)
        if subInfo.issection():
            typeName = subInfo.sectiontype.name
            if subInfo.ismulti():
                for subSect in value:
                    subLines = _subsectLines(
                        typeName, subSect.getSectionName() or '',
                        subSect, indent)
                    cfgLines.extend(subLines)
            else:
                sectName = subInfo.attribute
                subLines = _subsectLines(
                    typeName, (typeName != sectName) and sectName or '',
                    value, indent)
                cfgLines.extend(subLines)
        else:
            indentStr = INDENT * indent
            if subInfo.ismulti():
                for val in value:
                    cfgLines.append('%s%s %s' % (indentStr, key, val))
            else:
                cfgLines.append('%s%s %s' % (indentStr, key, value))
    return cfgLines

def _subsectLines(typeName, sectName, subSect, indent):
    cfgLines = []
    indentStr = INDENT * indent
    line = '%s<%s' % (indentStr, typeName) # start of subsection
    if sectName: # sectName may be empty
        line = '%s %s>' % (line, sectName)
    else:
        line += '>'
    cfgLines.append(line)
    subLines = sectionLines(subSect, indent + 1) # recurse
    cfgLines.extend(subLines)
    cfgLines.append('%s</%s>' % (indentStr, typeName)) # end of subsection
    return cfgLines

def main():
    thisDir = os.path.dirname(os.path.abspath(__file__))
    schemaPath = os.path.join(thisDir, 'schema.xml')
    schema = ZConfig.loadSchema(schemaPath)
    cfgReadPath = os.path.join(thisDir, 'read.conf')
    cfg, _ = ZConfig.loadConfig(schema, cfgReadPath)
    cfgLines = sectionLines(cfg)
    cfgLines.append('') # add the final newline
    cfgWritePath = os.path.join(thisDir, 'write.conf')
    try:
        fd = file(cfgWritePath, 'w')
        fd.write('\n'.join(cfgLines))
    finally:
        fd.close()
    return

if __name__ == '__main__':
    main()
